package com.nsgk.agentcentersdk.api;

import cn.hutool.core.util.URLUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.nsgk.agentcentersdk.core.NSConstants;
import com.nsgk.agentcentersdk.core.NSProtocol;
import com.nsgk.agentcentersdk.core.NSReportObject;
import com.nsgk.agentcentersdk.message.NSMessage;
import com.nsgk.agentcentersdk.err.NSAssert;
import com.nsgk.agentcentersdk.err.NSErrGlobal;
import com.nsgk.agentcentersdk.err.NSErrno;
import com.nsgk.agentcentersdk.network.NSNetworkManager;
import com.nsgk.agentcentersdk.network.NSNetworkRequest;
import com.nsgk.agentcentersdk.network.NSNetworkResponse;
import com.nsgk.agentcentersdk.utility.NSCrypto;
import lombok.Data;
import lombok.experimental.Accessors;

// SDK客户端
/*
请求:
    <host>:<port>/<api>?identifier=<identifier>&protocol=<protocol>&timestamp=<timestamp>
        header: sign
        body: 加密json
响应:
    body:
        code: 200
        msg: ""
        timestamp:
        data: "" // 加密json
 */
@Data
@Accessors(chain = true)
public final class NSSDKClient
{
    private final String host;
    private final Short port;
    private final String identifier;
    private final String publicKey;
    private int timeout;

    NSSDKClient(String host, Short port, String identifier, String publicKey, int timeout)
    {
        this.host = host;
        this.port = port;
        this.identifier = identifier;
        this.publicKey = publicKey;
        this.timeout = timeout;
    }

    // 发送请求
    public <T extends NSMessage> NSApiResult Send(int protocol, T object)
    {
        String url;
        NSReportObject<T> reportObject;
        NSNetworkRequest request;

        NSAssert.NotNull(object, NSErrno.ERRNO_CLI_DATA_MISSING);
        NSAssert.NotEmpty(identifier, NSErrno.ERRNO_CLI_PARAMETER_MISSING);
        NSErrGlobal.ASSERT(NSProtocol.IsValid(protocol), NSErrno.ERRNO_SYS_INVALID_PROTOCOL);
        url = NSApi.NS_API_INTERFACE;
        reportObject = new NSReportObject<>(identifier, protocol, object);
        request = new NSNetworkRequest();
        request.setUrl(BuildUrl(url));
        request.setTimeout(timeout);
        WriteDataObject(request, reportObject);
        request.AddHeader("Content-Type", "text/plain");
        NSNetworkResponse response = NSNetworkManager.Post(request);
        NSErrGlobal.ASSERT(response.IsSuccess(), NSErrno.ERRNO_SYS_HTTP);
        String json = response.getData();
        return FromJSON(json);
    }

    private String BuildUrl(String url)
    {
        return URLUtil.completeUrl(host + (null != port ? ":" + port : ""), url);
    }

    private <T> void WriteDataObject(NSNetworkRequest request, NSReportObject<T> object)
    {
        String dataStr;

        request.ClearHeaders().AddHeader(NSConstants.HEADER_SIGN, object.getSign())
               .AddHeader(NSConstants.HEADER_IDENTIFIER, object.getIdentifier())
               .AddHeader(NSConstants.HEADER_TIMESTAMP, object.getTimestamp())
               ;
        request.AddQuery(NSConstants.QUERY_PROTOCOL, object.getProtocol())
        ;
        dataStr = object.getDataStr();
        try
        {
            dataStr = NSCrypto.RSAEncrypt_publicKey(dataStr, publicKey);
            NSAssert.NotEmpty(dataStr, NSErrno.ERRNO_SYS_ENCRYPT_FAIL);
        }
        catch(Exception e)
        {
            e.printStackTrace();
            NSErrGlobal.Throw(NSErrno.ERRNO_SYS_ENCRYPT_FAIL);
            return;
        }
        request.setData(dataStr);
    }

    NSApiResult FromJSON(String str)
    {
        JSONObject jsonObject;

        jsonObject = JSON.parseObject(str);
        return new NSApiResult(
                jsonObject.getInteger("code"),
                jsonObject.getString("msg"),
                UnwrapClientResponseData(jsonObject.get("data")),
                jsonObject.containsKey("timestamp") ? jsonObject.getLong("timestamp") : null,
                jsonObject.containsKey("errno") ? jsonObject.getInteger("errno") : NSErrno.ERRNO_OK
        );
    }

    private Object UnwrapClientResponseData(Object data)
    {
        if(null != data)
        {
            if(data instanceof String)
                data = NSCrypto.RSADecrypt_publicKey((String)data, publicKey);
        }
        return data;
    }
}
