package com.nsgk.agentcentersdk.api;

import com.alibaba.fastjson2.JSON;
import com.nsgk.agentcentersdk.core.NSConstants;
import com.nsgk.agentcentersdk.core.NSProtocol;
import com.nsgk.agentcentersdk.core.NSReportObject;
import com.nsgk.agentcentersdk.message.NSMessage;
import com.nsgk.agentcentersdk.err.NSAssert;
import com.nsgk.agentcentersdk.err.NSErrGlobal;
import com.nsgk.agentcentersdk.err.NSErrno;
import com.nsgk.agentcentersdk.utility.NSArr;
import com.nsgk.agentcentersdk.utility.NSCrypto;
import com.nsgk.agentcentersdk.utility.NSHttp;

import javax.servlet.http.HttpServletRequest;

// SDK服务端
public final class NSSDKServer
{
    private final String privateKey;

    public NSSDKServer(String privateKey)
    {
        this.privateKey = privateKey;
    }

    public static int GetProtocol(HttpServletRequest request)
    {
        try
        {
            return Integer.parseInt(request.getParameter(NSConstants.QUERY_PROTOCOL));
        }
        catch(Exception e)
        {
            e.printStackTrace();
            return NSProtocol.NS_PROTOCOL_INVALID;
        }
    }

    // 解析请求
    public <T extends NSMessage> NSReportObject<T> Recv(HttpServletRequest request, Class<T> clazz)
    {
        NSReportObject<T> res;
        String sign;
        String data;

        sign = request.getHeader(NSConstants.HEADER_SIGN);
        NSAssert.NotEmpty(sign, NSErrno.ERRNO_CLI_SIGN_MISSING);
        data = NSHttp.GetRequestBody(request);
        NSAssert.NotEmpty(data, NSErrno.ERRNO_CLI_DATA_MISSING);
        res = new NSReportObject<>();
        res.SetTimestampStr(request.getHeader(NSConstants.HEADER_TIMESTAMP))
                .setIdentifier(request.getHeader(NSConstants.HEADER_IDENTIFIER))
                .SetProtocolStr(request.getParameter(NSConstants.QUERY_PROTOCOL))
        ;
        try
        {
            data = NSCrypto.RSADecrypt_privateKey(data, privateKey);
            NSAssert.NotEmpty(data, NSErrno.ERRNO_SYS_DECRYPT_FAIL);
            res.SetDataStr(data, clazz);
            NSErrGlobal.ASSERT(res.getData().Normalized() == 0, NSErrno.ERRNO_SYS_INVALID_FIELD);
        }
        catch(Exception e)
        {
            e.printStackTrace();
            return NSErrGlobal.ThrowAndReturnNull(NSErrno.ERRNO_SYS_DECRYPT_FAIL);
        }
        if(!res.CheckSign(sign))
        {
            return NSErrGlobal.ThrowAndReturn(NSErrno.ERRNO_SYS_INVALID_SIGN, null);
        }
        if(!res.CheckTime(ServerTime(), NSConstants.MAX_TIME_DIFF * 1000))
        {
            return NSErrGlobal.ThrowAndReturn(NSErrno.ERRNO_SYS_TIME_DIFF, null);
        }
        res.setSign(sign);
        return res;
    }

    private static long ServerTime()
    {
        return System.currentTimeMillis();
    }

    // 响应客户端
    public NSApiResult Resp(boolean success, String msg, Object...data)
    {
        return new NSApiResult(success, msg, WrapClientResponseData(data));
    }

    public NSApiResult Resp(int errno, String msg, Object...data)
    {
        return new NSApiResult(errno == NSErrno.ERRNO_OK, msg, WrapClientResponseData(data), errno);
    }

    public NSApiResult Success(String msg, Object...data)
    {
        return NSApiResult.Success(msg, WrapClientResponseData(data));
    }

    public NSApiResult Fail(int errno, String msg, Object...data)
    {
        return NSApiResult.Error(errno, msg, WrapClientResponseData(data));
    }

    private Object WrapClientResponseData(Object...data)
    {
        Object o = NSArr.DefParm(data);
        if(null != o)
        {
            if(!(o instanceof String))
                o = NSCrypto.RSAEncrypt_privateKey(JSON.toJSONString(o), privateKey);
        }
        return o;
    }
}
