package com.ruoyi.common.utils.handler;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.lang.Assert;
import com.ruoyi.common.utils.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.annotation.PreDestroy;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
public final class HandlerEngine
{
    private static final Logger               logger        = LoggerFactory.getLogger("HandlerEngine");
    private static final List<Object>         _listenerList = new ArrayList<>();
    private static final Map<String, Handler> _handlerMap   = new HashMap<>();

    public static Object Post(Object...message)
    {
        Handler handler = _handlerMap.values().stream().filter((x) -> {
            String err = x.Match(message);
            //System.err.println(err);
            return null == err;
        }).findFirst().orElse(null);
        Assert.notNull(handler, "没有对应处理器被注册");
        return handler.Invoke(message);
    }

    @SuppressWarnings("unchecked")
    public static <T> T PostT(Object...message)
    {
        return (T) Post(message);
    }

    public static Object Call(String protocol, Object...message)
    {
        Handler handler = _handlerMap.get(protocol);
        Assert.notNull(handler, "协议对应的处理器不存在: {}", protocol);
        return handler.Invoke(message);
    }

    @SuppressWarnings("unchecked")
    public static <T> T CallT(String protocol, Object...message)
    {
        return (T) Call(protocol, message);
    }

    public static void Register(Object listener)
    {
        if(null == listener)
            return;
        synchronized(_listenerList)
        {
            if(_listenerList.contains(listener))
                return;
            List<Handler> handlers = parseObject(listener);
            if(CollectionUtil.isEmpty(handlers))
                return;
            if(handlers.stream().anyMatch((x) -> _handlerMap.containsKey(x.protocol)))
                return;
            for(Handler handler : handlers)
            {
                _handlerMap.put(handler.protocol, handler);
                logger.info("Register -> " + handler);
            }
            _listenerList.add(listener);
            logger.info("HandlerEngine::Register -> " + listener);
        }
    }

    public static void Unregister(Object listener)
    {
        if(null == listener)
            return;
        synchronized(_listenerList)
        {
            if(!_listenerList.contains(listener))
                return;
            List<String> rm = new ArrayList<>();
            _handlerMap.forEach((k, v) -> {
                if(v.handler == listener)
                {
                    rm.add(k);
                    logger.info("Unregister -> " + v);
                }
            });
            rm.forEach(_handlerMap::remove);
            _listenerList.remove(listener);
            logger.info("HandlerEngine::Unregister -> " + listener);
        }
    }

    @PreDestroy
    public static void UnregisterAll()
    {
        synchronized(_listenerList)
        {
            logger.info("HandlerEngine::UnregisterAll -> " + _listenerList.size());
            _handlerMap.clear();
            _listenerList.clear();
        }
    }

    private static List<Handler> parseObject(Object obj)
    {
        if(null == obj)
            return null;
        Class<?> clazz = obj.getClass();
        return parseObject(obj, clazz);
    }

    private static List<Handler> parseObject(Object obj, Class<?> clazz)
    {
        if(null == clazz)
            return null;
        List<Handler> list = new ArrayList<>();
        Method[] declaredMethods = clazz.getDeclaredMethods();
        for(Method method : declaredMethods)
        {
            CallbackHandler annotation = method.getAnnotation(CallbackHandler.class);
            if(null == annotation)
                continue;
            String protocol = annotation.protocol();
            if(StringUtils.isEmpty(protocol))
                protocol = GenProtocol();
            Handler handler = new Handler(protocol, obj, method);
            list.add(handler);
        }
        Class<?> superclass = clazz.getSuperclass();
        if(null != superclass && !Object.class.equals(superclass))
        {
            List<Handler> handlers = parseObject(obj, superclass);
            if(CollectionUtil.isNotEmpty(handlers))
                list.addAll(handlers);
        }
        return list;
    }

    private static int _autoProtocolId = 0;
    private static String GenProtocol()
    {
        return "__AUTO_GEN_PROTOCOL_" + _autoProtocolId++;
    }
}
